% One-dimensional equation of heat transfer for inhomogeneous medium

function [uu]=heat_1d_nun(ud,roc,a,nx,h,time,tau,b,gamma,bl,br,fbl,fbr,fs)

th=tau/(h*h);
if gamma > 0. 
    
% Implicit scheme
    if gamma < 0.5
        if b > 1./(2.-4.*gamma)
            disp('scheme is unstable'); return;
        end    
    end    
    p(1:nx)=zeros(1,nx); s(1:nx)=zeros(1,nx); r(1:nx)=zeros(1,nx); d(1:nx)=zeros(1,nx);
    alfa(1:nx)=zeros(1,nx); beta(1:nx)=zeros(1,nx);
    if bl == 0
        s(1)=1.; r(1)=0.; d(1)=feval(fbl,time);
    else
        s(1)=1.+2.*gamma*a(1)*th/roc(1); r(1)=-2.*gamma*a(1)*th/roc(1);
        d(1)=(1.-2.*(1.-gamma)*a(1)*th/roc(1))*ud(1)+2.*(1.-gamma)*a(1)*th*ud(2)/roc(1)-...
                 2.*h*th*(gamma*feval(fbl,time)+(1.-gamma)*feval(fbl,time-tau))/roc(1);
    end
    for n=2:nx-1
        x=(n-1)*h;
        p(n)=-gamma*a(n-1)*th/roc(n);
        s(n)=1.+gamma*(a(n-1)+a(n))*th/roc(n); r(n)=-gamma*a(n)*th/roc(n);
        d(n)=ud(n)*(1.-(a(n)+a(n-1))*th*(1.-gamma)/roc(n))+...
                          (1.-gamma)*th*(a(n)*ud(n+1)+a(n-1)*ud(n-1))/roc(n)+tau*feval(fs,x,time);
    end
    if br == 0
        p(nx)=0.; s(nx)=1.; d(nx)=feval(fbr,time);
    else
        s(nx)=1.+2.*gamma*a(nx-1)*th/roc(nx); p(nx)=-2.*gamma*a(nx-1)*th/roc(nx);
        d(nx)=(1.-2.*a(nx-1)*th*(1.-gamma)/roc(nx))*ud(nx)+...
                   2.*a(nx-1)*th*(1.-gamma)*ud(nx-1)/roc(nx)+...
                   2.*h*th*(gamma*feval(fbr,time)+(1.-gamma)*feval(fbr,time-tau))/roc(nx);
    end                  
    alfa(1)=-r(1)/s(1); beta(1)=d(1)/s(1);
    for n=2:nx
        aa=s(n)+p(n)*alfa(n-1);
        alfa(n)=-r(n)/aa; beta(n)=(d(n)-p(n)*beta(n-1))/aa;
    end
    uu(nx)=beta(nx)/aa;
    for n=nx-1:-1:1
        uu(n)=alfa(n)*uu(n+1)+beta(n);
    end
    return;
else    
    
% Explicit scheme
    if b > 0.5
        disp('scheme is unstable'); return;
    end    
    if bl == 0
        uu(1)=feval(fbl,time);
    else
         uu(1)=(1.-2.*a(1)*th/roc(1))*ud(1)+2.*a(1)*th*ud(2)/roc(1)-...
                    2.*h*th*feval(fbl,time-tau)/roc(1);
    end
    for n=2:nx-1
        x=(n-1)*h;
        uu(n)=ud(n)*(1.-(a(n)+a(n-1))*th/roc(n))+(a(n)*ud(n+1)+a(n-1)*ud(n-1))*th/roc(n)+...
                  tau*feval(fs,x,time);
    end
    if br == 0
        uu(nx)=feval(fbr,time);
    else
        uu(nx)=(1.-2.*a(nx)*th/roc(nx))*ud(nx)+2.*a(nx-1)*th*ud(nx-1)/roc(nx)+...
                     2.*h*th*feval(fbr,time-tau)/roc(nx);
    end
    return;
end