% Dissipation and dispersion properties of various schemes

function [dis,cg]=properties(kh,r,method)

switch method
% scheme "cross"    
case 'crs'
    c=cos(0.5*kh); s=sin(0.5*kh);
    dis=0.; cg=c/sqrt(1.-r*r*s*s);
% Lax-Wendroff scheme   
case 'lxw'
    c=cos(kh); s=sin(kh); rr=r*r;
    dis=-0.5*log((1.-rr+rr*c)^2+rr*s*s);
    cg =(rr+(1.-r)*c)/((1.-rr+rr*c)^2+rr*s*s);
% scheme of P.Roe
case 'roe'
   c=cos(0.5*kh); s=sin(0.5*kh); ra=1.-2.*r;
   dis=0.; cg=1.+(s/c)^2;
   if r > 0.
       cg=0.5*(1.-ra*c/sqrt(1.-ra*ra*s*s))/r;
   end
% Godunov scheme of the first order   
case 'god'
   c=cos(kh); s=sin(kh); rr=r*r;
   dis=-0.5*log((1.-r)^2+rr+2.*r*(1.-r)*c);
   cg =(r+(1.-r)*c)/((1.-r)^2+rr+2.*r*(1.-r)*c);
% PEM scheme   
case 'pem'
   c=cos(0.5*kh); s=sin(0.5*kh);
   dis=0.; cg=c;
   z1=exp(-1.); z2=1.-z1; aa=z2*(3.*z1-1.);
   if r > 0.
       z3=1.-exp(-r); z4=z1*(exp(r)-1.);
       b1=(z1*z3+(1.-2.*z1)*z4-z2*z2*r)/aa;
       b2=(r*z2*(1.+z1)-z2*(z3+z4))/aa;
       b3=(z1*z4+(1.-2.*z1)*z3-z2*z2*r)/aa;
       g1=1.-2.*b2*s*s-4.*b3*s*s*c;
       g2=2.*b2*s*c+2.*b3*s*c*c-2.*b3*s*s*s+2.*b1*s;
       g3=-2.*b2*s*c-4.*b3*s*c*c+2.*b3*s*s*s;
       g4=(b2+b3*c)*(c*c-s*s)+(b1-3.*b3*s*s)*c;
       dis=-0.5*log(g1*g1+g2*g2);
       cg =(g1*g4-g2*g3)/(r*(g1*g1+g2*g2));
   end
otherwise
    disp('Unknown method.');
end